const path = require('path');
const jsondiffpatch = require('jsondiffpatch');

const isExtractTextGarbage = /\.css\..*\.js/;
const isSourcemap = /\.map$/;

function mapStatsToEntriesWithSize(assets) {
    return assets.reduce((mem, asset) => {
        if (isExtractTextGarbage.test(asset.name)) {
            return mem;
        }

        if (isSourcemap.test(asset.name)) {
            return mem;
        }

        const entryName = asset.chunkNames[0];

        return Object.assign(
            {}, 
            mem, 
            {
                [entryName]: {
                    size: asset.size,
                    name: asset.name
                }
            }
            );
    }, {});
}

function diffToFileSizeChange(diff) {
    return Object.keys(diff).reduce((mem, asset) => {
        if (!diff[asset].size) {
            return mem;
        }
        mem[asset] = diff[asset].size.reduce((mem, val, i) => i === 0 ? mem - val : mem + val, 0);

        return mem;
    }, {});
}

module.exports = function (base, changed) {
    const resultBase = mapStatsToEntriesWithSize(base.assets);
    const resultChanged = mapStatsToEntriesWithSize(changed.assets);

    const diff = jsondiffpatch.diff(resultBase, resultChanged);

    if (!diff) {
        return {};
    }

    return diffToFileSizeChange(diff);
}
